; ###################################################################################################
; #                                       JV-ODE Truck Demo                                         #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE Car Demo - Truck & Trailer"

Graphics3D 800,600,0,2

Global CMass#=400.0				; ### Cab Mass
Global WMass#=14.0				; ### Wheel Mass
Global TMass#=800.0				; ### Trailer Mass

Global WorldERP#=1.0			; ### World Error Correction
Global WorldFriction#=80.0		; ### World Friction

Global Torque#=48.0				; ### Joint Torque
Global SuspensionHS#=0.01		; ### Suspension Hardness/Softness (Higher=Softer - Lower=Harder)

Global Force#=0.0
Global Steer#=0.0

Global Cab
Global CGeom
Global CMesh

Global Trailer
Global TGeom
Global TMesh

Global TruckStartY=4

Dim Wheel(6)
Dim WGeom(6)
Dim Joint(6)

Dim TWheel(6)
Dim TWGeom(6)
Dim TJoint(6)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dWorldSetERP(World,WorldERP)

dContactSetMode(dContactSlip1)
dContactSetMu(WorldFriction)

; ### Create Cab

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
Cab=ode\body
dBodySetPosition(ode\body,0,TruckStartY,0)
dBodySetRotation(ode\body,0,0,0)
mass=dMassCreate()
dMassSetBoxTotal(mass,CMass,3,1,8)
dBodySetMass(ode\body,mass)
dMassDestroy(mass)
ode\geom=dCreateBox(Space,3,1,8)
CGeom=ode\geom
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
CMesh=ode\mesh
ScaleMesh ode\mesh,1.5,0.5,4
EntityColor ode\mesh,40,80,255

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,3,2,4)
dGeomSetBody(ode\geom,Cab)
dGeomSetOffsetPosition(ode\geom,0,1.5,2)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,1.5,1,2
EntityColor ode\mesh,40,80,255
EntityAlpha ode\mesh,0.7

; ### Create Wheels

For count=1 To 6
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	Wheel(count)=ode\body
	dBodySetPosition(ode\body,0,0,0)
	dBodySetRotation(ode\body,0,0,90)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,WMass,0.7)
	dBodySetMass(ode\body,mass)
	dMassDestroy(mass)
	ode\geom=dCreateSphere(Space,0.7)
	WGeom(count)=ode\geom
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCylinder()
	ScaleMesh ode\mesh,0.7,0.25,0.7
	EntityColor ode\mesh,255,255,255
Next

dBodySetPosition(Wheel(1),-2,TruckStartY-0.5,+3)
dBodySetPosition(Wheel(2),+2,TruckStartY-0.5,+3)
dBodySetPosition(Wheel(3),-2,TruckStartY-0.5,-1.5)
dBodySetPosition(Wheel(4),+2,TruckStartY-0.5,-1.5)
dBodySetPosition(Wheel(5),-2,TruckStartY-0.5,-3)
dBodySetPosition(Wheel(6),+2,TruckStartY-0.5,-3)

; ### Create Trailer

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
Trailer=ode\body
dBodySetPosition(ode\body,0,TruckStartY+2,-10.5)
dBodySetRotation(ode\body,0,0,0)
mass=dMassCreate()
dMassSetBoxTotal(mass,TMass,3,3,18)
dBodySetMass(ode\body,mass)
dMassDestroy(mass)
ode\geom=dCreateBox(Space,3,3,18)
TGeom=ode\geom
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
TMesh=ode\mesh
ScaleMesh ode\mesh,1.5,1.5,9
EntityColor ode\mesh,20,40,128
EntityAlpha ode\mesh,0.7

; ### Create Trailer Wheels

For count=1 To 6
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	TWheel(count)=ode\body
	dBodySetPosition(ode\body,0,0,0)
	dBodySetRotation(ode\body,0,0,90)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,WMass,0.7)
	dBodySetMass(ode\body,mass)
	dMassDestroy(mass)
	ode\geom=dCreateSphere(Space,0.7)
	TWGeom(count)=ode\geom
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCylinder()
	ScaleMesh ode\mesh,0.7,0.25,0.7
	EntityColor ode\mesh,255,255,255
Next

dBodySetPosition(TWheel(1),-2,TruckStartY-0.5,-18)
dBodySetPosition(TWheel(2),+2,TruckStartY-0.5,-18)
dBodySetPosition(TWheel(3),-2,TruckStartY-0.5,-16.5)
dBodySetPosition(TWheel(4),+2,TruckStartY-0.5,-16.5)
dBodySetPosition(TWheel(5),-2,TruckStartY-0.5,-15)
dBodySetPosition(TWheel(6),+2,TruckStartY-0.5,-15)

; ### Create Objects

SeedRnd MilliSecs()

For spheres=1 To 20
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode\body,0,0,0)
	ode\geom=dCreateSphere(Space,4)
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateSphere()
	ScaleMesh ode\mesh,4,4,4
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

For capsules=1 To 20
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode\body,90,0,0)
	ode\geom=dCreateCapsule(Space,1,8)
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCylinder()
	ScaleMesh ode\mesh,1,5,1
	RotateMesh ode\mesh,90,0,0	; ### Align Blitz3D Cylinder To ODE Capsule
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global CameraPivot=CreatePivot()

Global Camera=CreateCamera(CameraPivot)
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

; ### Blitz3D Static Object (White)

Global Blitz3DObject=CreateCube()
ScaleMesh Blitz3DObject,1,4,20
PositionEntity Blitz3DObject,10,5,40
RotateEntity Blitz3DObject,-10,8,4
EntityColor Blitz3DObject,255,255,255

; ### ODE Static Object (Blue)

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,2,8,40)
dGeomSetPosition(ode\geom,20,5,40)
dGeomSetRotation(ode\geom,-10,8,4)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,1,4,20
EntityColor ode\mesh,0,100,200

; ### ODE Static Object (Red)

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,18,0.2,40)
dGeomSetPosition(ode\geom,-20,4.9,40)
dGeomSetRotation(ode\geom,-14,0,0)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,9,0.1,20
EntityColor ode\mesh,200,0,100

; ###################################################################################################

SetupTruck()

While Not KeyHit(1)

	UpdateKeys()

	UpdateTruck()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	UpdateCam()

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 0,50,"Force:"+Force
	Text 0,65,"Torque:"+Torque
	Text 640,0,"A - Accelerate"
	Text 640,15,"Z - Brake/Reverse"
	Text 640,30,"Arrows - Steering"
	Text 640,45,"Space - Reset Truck"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function SetupTruck()

For count=1 To 6
	Joint(count)=dJointCreateHinge2(World,0)
	dJointAttach(Joint(count),Cab,Wheel(count))
	dJointSetHinge2Anchor(Joint(count),dBodyGetPositionX(Wheel(count)),dBodyGetPositionY(Wheel(count)),dBodyGetPositionZ(Wheel(count)))
	dJointSetHinge2Axis1(Joint(count),0,1,0)
	dJointSetHinge2Axis2(Joint(count),-1,0,0)
	dJointSetHinge2Param(Joint(count),dParamSuspensionERP,0.8)
	dJointSetHinge2Param(Joint(count),dParamSuspensionCFM,SuspensionHS)
	If count>2
		dJointSetHinge2Param(Joint(count),dParamLoStop,0)
		dJointSetHinge2Param(Joint(count),dParamHiStop,0)
		End If
Next

For count=1 To 6
	TJoint(count)=dJointCreateHinge2(World,0)
	dJointAttach(TJoint(count),Trailer,TWheel(count))
	dJointSetHinge2Anchor(TJoint(count),dBodyGetPositionX(TWheel(count)),dBodyGetPositionY(TWheel(count)),dBodyGetPositionZ(TWheel(count)))
	dJointSetHinge2Axis1(TJoint(count),0,1,0)
	dJointSetHinge2Axis2(TJoint(count),-1,0,0)
	dJointSetHinge2Param(TJoint(count),dParamSuspensionERP,0.8)
	dJointSetHinge2Param(TJoint(count),dParamSuspensionCFM,SuspensionHS)
	dJointSetHinge2Param(TJoint(count),dParamLoStop,0)
	dJointSetHinge2Param(TJoint(count),dParamHiStop,0)
Next

AJoint=dJointCreateHinge(World,0)
dJointAttach(AJoint,Trailer,Cab)
dJointSetHingeAnchor(AJoint,dBodyGetPositionX(Cab),dBodyGetPositionY(Cab),dBodyGetPositionZ(Cab)-2)
dJointSetHingeAxis(AJoint,0,1,0)
dJointSetHingeParam(AJoint,dParamSuspensionERP,0.8)
dJointSetHingeParam(AJoint,dParamSuspensionCFM,SuspensionHS)
dJointSetHingeParam(AJoint,dParamLoStop,-2)
dJointSetHingeParam(AJoint,dParamHiStop,2)

End Function

; ###################################################################################################

Function UpdateKeys()

If KeyDown(30)=1
	Force=Force+0.08
	If Force>30 Then Force=30
	End If

If KeyDown(44)=1
	Force=Force-0.1
	If Force<-10 Then Force=-10
	If Force>0 Then Force=Force*0.97
	End If

If KeyDown(30)=0 And KeyDown(44)=0 Then Force=Force*0.99

If KeyDown(203)=1 Or KeyDown(205)=1
	If KeyDown(203)=1
		Steer=0.5
		Else
		Steer=-0.5
		End If
	Else
	Steer=0.0
	End If

If KeyHit(57)=1
	Force=0.0
	dBodySetRotation(Cab,dGeomGetPitch(CGeom),dGeomGetYaw(CGeom),0)
	dBodySetRotation(Trailer,dGeomGetPitch(TGeom),dGeomGetYaw(TGeom),0)
	End If

End Function

; ###################################################################################################

Function UpdateTruck()

For count=1 To 6
	dBodyEnable(Wheel(count))
	dBodyEnable(TWheel(count))
Next

dBodyEnable(Cab)
dBodyEnable(Trailer)

For count=1 To 6
	dJointSetHinge2Param(Joint(count),dParamVel2,Force)
	dJointSetHinge2Param(Joint(count),dParamFMax2,Torque)
Next

For count=1 To 2
	angle#=Steer-dJointGetHinge2Angle1(Joint(count))
	dJointSetHinge2Param(Joint(count),dParamVel,angle)
	dJointSetHinge2Param(Joint(count),dParamFMax,400)
Next

End Function

; ###################################################################################################

Function UpdateCam()

PositionEntity Camera,0,15,-25
RotateEntity Camera,30,0,0

PivotX#=Cos(EntityYaw(CMesh)-90)+EntityX(CMesh)
PivotY#=EntityY(CMesh)
PivotZ#=Sin(EntityYaw(CMesh)-90)+EntityZ(CMesh)

PositionEntity CameraPivot,PivotX,PivotY,PivotZ

PointEntity CameraPivot,CMesh

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################
