; ###################################################################################################
; #                                 JV-ODE - Collision Forces Demo                                  #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE - Collision Forces Demo"

Graphics3D 800,600,0,2

Dim Joint(10)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.95)
dContactSetMu(48)

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global Camera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,40,-30
RotateEntity Camera,30,0,0

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.5

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

AddObjects()

While Not KeyHit(1)

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 10,280,"Collision Force:"

	For count=1 To 10
		dJointGetFeedbackForce1(Joint(count))
		ShowBar(78+(count*58),320,dVectorY())
	Next

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function AddObjects()

For count=1 To 10
	; ### Body & Geom
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	ParentBody=ode\body
	dBodySetPosition(ode\body,-55+(10*count),45,30)
	dBodySetRotation(ode\body,0,0,0)
	dBodySetAutoDisableFlag(ode\body,0)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,0.5,4)
	dBodySetMass(ode\body,mass)
	dMassDestroy(mass)
	ode\geom=dCreateSphere(Space,4)
	dGeomContactSetMode(ode\geom,dContactBounce)
	dGeomContactSetBounce(ode\geom,count*0.1)
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateSphere()
	ScaleMesh ode\mesh,4,4,4
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7

	; ### Core Body
	CoreBody=dBodyCreate(World)
	dBodySetPosition(CoreBody,-55+(10*count),45,30)
	dBodySetRotation(CoreBody,0,0,0)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,0.5,4)
	dBodySetMass(CoreBody,mass)
	dMassDestroy(mass)
	dBodySetAutoDisableFlag(CoreBody,0)

	; ### Join Core & Parent
	Joint(count)=dJointCreateFixed(World,0)
	dJointAttach(Joint(count),CoreBody,ParentBody)
	dJointFeedbackEnable(Joint(count))
Next

End Function

; ###################################################################################################

Function ShowBar(xpos,ypos,yvec,size=8)

If yvec<0
	Color 240,0,140
	Else
	Color 0,240,140
	End If

Rect xpos,ypos,size,size+Abs(Int(yvec*2))

Color 255,255,255

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################