; ###################################################################################################
; #                             JV-ODE - Zombie Animated RagDoll Demo                               #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE - Zombie Animated RagDoll Demo"

Graphics3D 800,600,0,2

Global RagDollMax=3
Global ODEJointMax=14
Global MeshBoneMax=24

Dim RagDollMesh(RagDollMax)
Dim Bone(RagDollMax,MeshBoneMax)
Dim Node(RagDollMax,MeshBoneMax)

Dim SetBone(ODEJointMax)
Dim RDBody(RagDollMax,ODEJointMax)
Dim RDMesh(RagDollMax,ODEJointMax)
Dim RDJoint(RagDollMax,ODEJointMax)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactSlip1+dContactBounce)
dContactSetBounce(0.4)
dContactSetMu(2)

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global Camera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,9,-4
RotateEntity Camera,30,0,0

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.5

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

; ### Create RagDolls

SetupBones()			; ### Set Bones Used For ODE Joints

SetupNodes(1,0.5,1)		; ### RagDoll ID - Mesh Alpha - Show Nodes
CreateRagDoll(1,0)		; ### RagDoll ID - Geom Alpha

SetupNodes(2,0.5,1)		; ### RagDoll ID - Mesh Alpha - Show Nodes
CreateRagDoll(2,0.5)	; ### RagDoll ID - Geom Alpha

SetupNodes(3,1,0)		; ### RagDoll ID - Mesh Alpha - Show Nodes
CreateRagDoll(3,0)		; ### RagDoll ID - Geom Alpha

; ###################################################################################################

While Not KeyHit(1)

	UpdateControls()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	For count=1 To RagDollMax
		UpdateNodes(count)
	Next

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 300,0,"F1 - Drop RagDoll 1"
	Text 300,15,"F2 - Drop RagDoll 2"
	Text 300,30,"F3 - Drop RagDoll 3"
	Text 300,50,"Space - Throw RagDolls"
	Text 300,65,"Arrow Keys - Move Camera"
	Text 580,0,"F4 - Attach RagDoll 1"
	Text 580,15,"F5 - Attach RagDoll 2"
	Text 580,30,"F6 - Attach RagDoll 3"
	Text 580,50,"L Mouse Button - Push Body"
	Text 580,65,"R Mouse Button - Pull Body"
	Text 200,580,"Zombie 3D Model Available From www.psionic3d.co.uk"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function SetupBones()

SetBone(1)=10		; ### Head To Body (Upper)
SetBone(2)=8		; ### Body (Upper) To Body (Lower)
SetBone(3)=11		; ### Left Arm (Upper) To Body (Upper)
SetBone(4)=12		; ### Left Arm (Lower) To Left Arm (Upper)
SetBone(5)=13		; ### Left Hand To Left Arm (Lower)
SetBone(6)=14		; ### Right Arm (Upper) To Body (Upper)
SetBone(7)=15		; ### Right Arm (Lower) To Right Arm (Upper)
SetBone(8)=16		; ### Right Hand To Right Arm (Lower)
SetBone(9)=21		; ### Left Leg (Upper) To Body (Lower)
SetBone(10)=22		; ### Left Leg (Lower) To Left Leg (Upper)
SetBone(11)=23		; ### Left Foot To Left Leg (Lower)
SetBone(12)=17		; ### Right Leg (Upper) To Body (Lower)
SetBone(13)=18		; ### Right Leg (Lower) To Right Leg (Upper)
SetBone(14)=19		; ### Right Foot To Right Leg (Lower)

End Function

; ###################################################################################################

Function SetupNodes(rd,meshalpha#,shownode)

xp#=-8+(rd*4)
yp#=0.3
zp#=4

RagDollMesh(rd)=LoadAnimMesh("media\zombie.b3d")

ScaleEntity RagDollMesh(rd),0.45,0.45,0.45
PositionEntity RagDollMesh(rd),xp,yp,zp
EntityColor RagDollMesh(rd),255,255,255
EntityAlpha RagDollMesh(rd),meshalpha

Restore NodeData

For count=1 To MeshBoneMax
	Read NodeInfo
	Bone(rd,count)=FindChild(RagDollMesh(rd),"Joint"+count)
	Node(rd,count)=CreateSphere(8,Bone(rd,count))
	ScaleEntity Node(rd,count),0.50,0.50,0.50
	EntityColor Node(rd,count),200,200,200
	EntityShininess Node(rd,count),0.7
	If NodeInfo=0
		ScaleEntity Node(rd,count),0.24,0.24,0.24
		EntityAlpha Node(rd,count),0
		End If
	If NodeInfo=1
		ScaleEntity Node(rd,count),0.24,0.24,0.24
		EntityColor Node(rd,count),count*5,80,200
		End If
	If NodeInfo=2
		ScaleEntity Node(rd,count),0.30,0.30,0.30
		EntityColor Node(rd,count),0,200,80
		End If
	If shownode=0 Then EntityAlpha Node(rd,count),0
Next

End Function

; ###################################################################################################

Function CreateRagDoll(rd,geomalpha#)

xp#=EntityX(RagDollMesh(rd))
yp#=EntityY(RagDollMesh(rd))+7.7
zp#=EntityZ(RagDollMesh(rd))

; ### RagDoll Dimensions

HeadRad#=0.45
BodyUX#=1.2
BodyUY#=1.8
BodyUZ#=0.65
BodyLX#=1.2
BodyLY#=0.95
BodyLZ#=0.65
ArmURad#=0.2
ArmULen#=1.25
ArmLRad#=0.2
ArmLLen#=1.25
LegURad#=0.3
LegULen#=1.6
LegLRad#=0.27
LegLLen#=2.5
FeetX#=0.4
FeetY#=0.18
FeetZ#=0.85
HandX#=0.16
HandY#=0.6
HandZ#=0.34

; ### Create Head

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,0)=ode\body
dBodySetPosition(ode\body,xp,yp,zp-0.1)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateSphere(Space,HeadRad)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateSphere()
RDMesh(rd,0)=ode\mesh
ScaleMesh ode\mesh,HeadRad,HeadRad,HeadRad
EntityColor ode\mesh,255,255,255
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Body (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,1)=ode\body
dBodySetPosition(ode\body,xp,yp-1.7,zp+0.05)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,BodyUX,BodyUY,BodyUZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
RDMesh(rd,1)=ode\mesh
ScaleMesh ode\mesh,BodyUX*0.5,BodyUY*0.5,BodyUZ*0.5
EntityColor ode\mesh,0,200,200
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Body (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,2)=ode\body
dBodySetPosition(ode\body,xp,yp-3.05,zp+0.05)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,BodyLX,BodyLY,BodyLZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
RDMesh(rd,2)=ode\mesh
ScaleMesh ode\mesh,BodyLX*0.5,BodyLY*0.5,BodyLZ*0.5
EntityColor ode\mesh,0,100,100
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Left Arm (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,3)=ode\body
dBodySetPosition(ode\body,xp+0.95,yp-1.65,zp+0.15)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmURad,ArmULen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,0,0,14)
ode\mesh=CreateCylinder()
RDMesh(rd,3)=ode\mesh
ScaleMesh ode\mesh,ArmURad,ArmULen*0.5,ArmURad
EntityColor ode\mesh,255,0,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Left Arm (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,4)=ode\body
dBodySetPosition(ode\body,xp+1.22,yp-2.85,zp+0.15)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmLRad,ArmLLen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,0,0,10)
ode\mesh=CreateCylinder()
RDMesh(rd,4)=ode\mesh
ScaleMesh ode\mesh,ArmLRad,ArmLLen*0.5,ArmLRad
EntityColor ode\mesh,125,0,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Left Hand

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,5)=ode\body
dBodySetPosition(ode\body,xp+1.32,yp-3.75,zp+0.1)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,HandX,HandY,HandZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
RDMesh(rd,5)=ode\mesh
ScaleMesh ode\mesh,HandX*0.5,HandY*0.5,HandZ*0.5
EntityColor ode\mesh,255,255,255
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Right Arm (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,6)=ode\body
dBodySetPosition(ode\body,xp-0.95,yp-1.65,zp+0.15)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmURad,ArmULen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,0,0,-14)
ode\mesh=CreateCylinder()
RDMesh(rd,6)=ode\mesh
ScaleMesh ode\mesh,ArmURad,ArmULen*0.5,ArmURad
EntityColor ode\mesh,0,255,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Right Arm (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,7)=ode\body
dBodySetPosition(ode\body,xp-1.22,yp-2.85,zp+0.15)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmLRad,ArmLLen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,0,0,-10)
ode\mesh=CreateCylinder()
RDMesh(rd,7)=ode\mesh
ScaleMesh ode\mesh,ArmLRad,ArmLLen*0.5,ArmLRad
EntityColor ode\mesh,0,125,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Right Hand

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,8)=ode\body
dBodySetPosition(ode\body,xp-1.32,yp-3.75,zp+0.1)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,0,0,0)		; ### Zero Geom Dimensions For Zombie 3D Model
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
RDMesh(rd,8)=ode\mesh
ScaleMesh ode\mesh,0,0,0				; ### Zero Mesh Dimensions For Zombie 3D Model
EntityColor ode\mesh,255,255,255
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Left Leg (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,9)=ode\body
dBodySetPosition(ode\body,xp+0.45,yp-4.3,zp+0.1)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegURad,LegULen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,0,0,7)
ode\mesh=CreateCylinder()
RDMesh(rd,9)=ode\mesh
ScaleMesh ode\mesh,LegURad,LegULen*0.5,LegURad
EntityColor ode\mesh,255,0,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Left Leg (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,10)=ode\body
dBodySetPosition(ode\body,xp+0.6,yp-6.25,zp+0.25)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegLRad,LegLLen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,-8,0,4)
ode\mesh=CreateCylinder()
RDMesh(rd,10)=ode\mesh
ScaleMesh ode\mesh,LegLRad,LegLLen*0.5,LegLRad
EntityColor ode\mesh,125,0,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Left Foot

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,11)=ode\body
dBodySetPosition(ode\body,xp+0.73,yp-7.55,zp+0.15)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,FeetX,FeetY,FeetZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
RDMesh(rd,11)=ode\mesh
ScaleMesh ode\mesh,FeetX*0.5,FeetY*0.5,FeetZ*0.5
EntityColor ode\mesh,255,255,255
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Right Leg (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,12)=ode\body
dBodySetPosition(ode\body,xp-0.45,yp-4.3,zp+0.1)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegURad,LegULen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,0,0,-7)
ode\mesh=CreateCylinder()
RDMesh(rd,12)=ode\mesh
ScaleMesh ode\mesh,LegURad,LegULen*0.5,LegURad
EntityColor ode\mesh,0,255,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Right Leg (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,13)=ode\body
dBodySetPosition(ode\body,xp-0.6,yp-6.25,zp+0.25)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegLRad,LegLLen)
dGeomSetBody(ode\geom,ode\body)
dGeomSetOffsetRotation(ode\geom,-8,0,-4)
ode\mesh=CreateCylinder()
RDMesh(rd,13)=ode\mesh
ScaleMesh ode\mesh,LegLRad,LegLLen*0.5,LegLRad
EntityColor ode\mesh,0,125,0
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Create Right Foot

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,14)=ode\body
dBodySetPosition(ode\body,xp-0.73,yp-7.55,zp+0.15)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,FeetX,FeetY,FeetZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
RDMesh(rd,14)=ode\mesh
ScaleMesh ode\mesh,FeetX*0.5,FeetY*0.5,FeetZ*0.5
EntityColor ode\mesh,255,255,255
EntityAlpha ode\mesh,geomalpha
EntityPickMode ode\mesh,2

; ### Static Joint To Head

RDJoint(rd,0)=dJointCreateBall(World,0)
dJointAttach(RDJoint(rd,0),0,RDBody(rd,0))
dJointSetBallAnchor(RDJoint(rd,0),xp,yp,zp)

; ### Head To Body (Upper)

RDJoint(rd,1)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,1),RDBody(rd,0),RDBody(rd,1))
dJointSetHingeAxis(RDJoint(rd,1),1,0,1)
dJointSetHingeAnchor(RDJoint(rd,1),EntityX(Node(rd,SetBone(1)),1),EntityY(Node(rd,SetBone(1)),1),EntityZ(Node(rd,SetBone(1)),1))
dJointSetHingeParam(RDJoint(rd,1),dParamLoStop,-0.7)
dJointSetHingeParam(RDJoint(rd,1),dParamHiStop,0.7)

; ### Body (Upper) To Body (Lower)

RDJoint(rd,2)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,2),RDBody(rd,1),RDBody(rd,2))
dJointSetHingeAxis(RDJoint(rd,2),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,2),EntityX(Node(rd,SetBone(2)),1),EntityY(Node(rd,SetBone(2)),1),EntityZ(Node(rd,SetBone(2)),1))
dJointSetHingeParam(RDJoint(rd,2),dParamLoStop,-0.25)
dJointSetHingeParam(RDJoint(rd,2),dParamHiStop,0.25)

; ### Left Arm (Upper) To Body (Upper)

RDJoint(rd,3)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,3),RDBody(rd,3),RDBody(rd,1))
dJointSetHingeAxis(RDJoint(rd,3),1,-1,1)
dJointSetHingeAnchor(RDJoint(rd,3),EntityX(Node(rd,SetBone(3)),1),EntityY(Node(rd,SetBone(3)),1),EntityZ(Node(rd,SetBone(3)),1))
dJointSetHingeParam(RDJoint(rd,3),dParamLoStop,-1.5)
dJointSetHingeParam(RDJoint(rd,3),dParamHiStop,1.5)

; ### Left Arm (Lower) To Left Arm (Upper)

RDJoint(rd,4)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,4),RDBody(rd,4),RDBody(rd,3))
dJointSetHingeAxis(RDJoint(rd,4),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,4),EntityX(Node(rd,SetBone(4)),1),EntityY(Node(rd,SetBone(4)),1),EntityZ(Node(rd,SetBone(4)),1))
dJointSetHingeParam(RDJoint(rd,4),dParamLoStop,0.0)
dJointSetHingeParam(RDJoint(rd,4),dParamHiStop,2.0)

; ### Left Hand To Left Arm (Lower)

RDJoint(rd,5)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,5),RDBody(rd,5),RDBody(rd,4))
dJointSetHingeAxis(RDJoint(rd,5),0,0,1)
dJointSetHingeAnchor(RDJoint(rd,5),EntityX(Node(rd,SetBone(5)),1),EntityY(Node(rd,SetBone(5)),1),EntityZ(Node(rd,SetBone(5)),1))
dJointSetHingeParam(RDJoint(rd,5),dParamLoStop,-0.1)
dJointSetHingeParam(RDJoint(rd,5),dParamHiStop,0.1)

; ### Right Arm (Upper) To Body (Upper)

RDJoint(rd,6)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,6),RDBody(rd,6),RDBody(rd,1))
dJointSetHingeAxis(RDJoint(rd,6),1,1,1)
dJointSetHingeAnchor(RDJoint(rd,6),EntityX(Node(rd,SetBone(6)),1),EntityY(Node(rd,SetBone(6)),1),EntityZ(Node(rd,SetBone(6)),1))
dJointSetHingeParam(RDJoint(rd,6),dParamLoStop,-1.5)
dJointSetHingeParam(RDJoint(rd,6),dParamHiStop,1.5)

; ### Right Arm (Lower) To Right Arm (Upper)

RDJoint(rd,7)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,7),RDBody(rd,7),RDBody(rd,6))
dJointSetHingeAxis(RDJoint(rd,7),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,7),EntityX(Node(rd,SetBone(7)),1),EntityY(Node(rd,SetBone(7)),1),EntityZ(Node(rd,SetBone(7)),1))
dJointSetHingeParam(RDJoint(rd,7),dParamLoStop,0.0)
dJointSetHingeParam(RDJoint(rd,7),dParamHiStop,2.0)

; ### Right Hand To Right Arm (Lower)

RDJoint(rd,8)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,8),RDBody(rd,8),RDBody(rd,7))
dJointSetHingeAxis(RDJoint(rd,8),0,0,1)
dJointSetHingeAnchor(RDJoint(rd,8),EntityX(Node(rd,SetBone(8)),1),EntityY(Node(rd,SetBone(8)),1),EntityZ(Node(rd,SetBone(8)),1))
dJointSetHingeParam(RDJoint(rd,8),dParamLoStop,-0.1)
dJointSetHingeParam(RDJoint(rd,8),dParamHiStop,0.1)

; ### Left Leg (Upper) To Body (Lower)

RDJoint(rd,9)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,9),RDBody(rd,9),RDBody(rd,2))
dJointSetHingeAxis(RDJoint(rd,9),1,0,-1)
dJointSetHingeAnchor(RDJoint(rd,9),EntityX(Node(rd,SetBone(9)),1),EntityY(Node(rd,SetBone(9)),1),EntityZ(Node(rd,SetBone(9)),1))
dJointSetHingeParam(RDJoint(rd,9),dParamLoStop,-1.0)
dJointSetHingeParam(RDJoint(rd,9),dParamHiStop,1.0)

; ### Left Leg (Lower) To Left Leg (Upper)

RDJoint(rd,10)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,10),RDBody(rd,10),RDBody(rd,9))
dJointSetHingeAxis(RDJoint(rd,10),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,10),EntityX(Node(rd,SetBone(10)),1),EntityY(Node(rd,SetBone(10)),1),EntityZ(Node(rd,SetBone(10)),1))
dJointSetHingeParam(RDJoint(rd,10),dParamLoStop,-2.0)
dJointSetHingeParam(RDJoint(rd,10),dParamHiStop,0.0)

; ### Left Foot To Left Leg (Lower)

RDJoint(rd,11)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,11),RDBody(rd,11),RDBody(rd,10))
dJointSetHingeAxis(RDJoint(rd,11),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,11),EntityX(Node(rd,SetBone(11)),1),EntityY(Node(rd,SetBone(11)),1),EntityZ(Node(rd,SetBone(11)),1))
dJointSetHingeParam(RDJoint(rd,11),dParamLoStop,-0.25)
dJointSetHingeParam(RDJoint(rd,11),dParamHiStop,0.25)

; ### Right Leg (Upper) To Body (Lower)

RDJoint(rd,12)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,12),RDBody(rd,12),RDBody(rd,2))
dJointSetHingeAxis(RDJoint(rd,12),1,0,1)
dJointSetHingeAnchor(RDJoint(rd,12),EntityX(Node(rd,SetBone(12)),1),EntityY(Node(rd,SetBone(12)),1),EntityZ(Node(rd,SetBone(12)),1))
dJointSetHingeParam(RDJoint(rd,12),dParamLoStop,-1.0)
dJointSetHingeParam(RDJoint(rd,12),dParamHiStop,1.0)

; ### Right Leg (Lower) To Right Leg (Upper)

RDJoint(rd,13)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,13),RDBody(rd,13),RDBody(rd,12))
dJointSetHingeAxis(RDJoint(rd,13),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,13),EntityX(Node(rd,SetBone(13)),1),EntityY(Node(rd,SetBone(13)),1),EntityZ(Node(rd,SetBone(13)),1))
dJointSetHingeParam(RDJoint(rd,13),dParamLoStop,-2.0)
dJointSetHingeParam(RDJoint(rd,13),dParamHiStop,0.0)

; ### Right Foot To Right Leg (Lower)

RDJoint(rd,14)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,14),RDBody(rd,14),RDBody(rd,13))
dJointSetHingeAxis(RDJoint(rd,14),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,14),EntityX(Node(rd,SetBone(14)),1),EntityY(Node(rd,SetBone(14)),1),EntityZ(Node(rd,SetBone(14)),1))
dJointSetHingeParam(RDJoint(rd,14),dParamLoStop,-0.25)
dJointSetHingeParam(RDJoint(rd,14),dParamHiStop,0.25)

End Function

; ###################################################################################################

Function UpdateControls()

If KeyDown(59)=1 Then dJointAttach(RDJoint(1,0),0,0)
If KeyDown(60)=1 Then dJointAttach(RDJoint(2,0),0,0)
If KeyDown(61)=1 Then dJointAttach(RDJoint(3,0),0,0)

If KeyDown(62)=1 Then dJointAttach(RDJoint(1,0),0,RDBody(1,0))
If KeyDown(63)=1 Then dJointAttach(RDJoint(2,0),0,RDBody(2,0))
If KeyDown(64)=1 Then dJointAttach(RDJoint(3,0),0,RDBody(3,0))

If KeyDown(57)=1
	dBodyAddForce RDBody(1,0),0,20,0
	dBodyAddForce RDBody(2,0),0,20,0
	dBodyAddForce RDBody(3,0),0,20,0
	End If

If KeyDown(200)=1 Then PositionEntity Camera,EntityX(Camera),EntityY(Camera),EntityZ(Camera)+0.3
If KeyDown(208)=1 Then PositionEntity Camera,EntityX(Camera),EntityY(Camera),EntityZ(Camera)-0.3
If KeyDown(203)=1 Then PositionEntity Camera,EntityX(Camera)-0.3,EntityY(Camera),EntityZ(Camera)
If KeyDown(205)=1 Then PositionEntity Camera,EntityX(Camera)+0.3,EntityY(Camera),EntityZ(Camera)

If MouseDown(1)=1
	CameraPick(Camera,MouseX(),MouseY())
	If PickedEntity()
		For ode.ODEGeom=Each ODEGeom
			If ode\mesh=PickedEntity()
				dBodyEnable(ode\body)
				dBodyAddForce ode\body,-20*PickedNX#(),-20*PickedNY#(),-20*PickedNZ#()
				End If
		Next
		End If
	End If

If MouseDown(2)=1
	CameraPick(Camera,MouseX(),MouseY())
	If PickedEntity()
		For ode.ODEGeom=Each ODEGeom
			If ode\mesh=PickedEntity()
				dBodyEnable(ode\body)
				dBodyAddForce ode\body,20*PickedNX#(),20*PickedNY#(),20*PickedNZ#()
				End If
		Next
		End If
	End If

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################

Function UpdateNodes(rd)

; ### Align Bones To ODE Joints

Skip=1

For count=1 To ODEJointMax
	If count=3 Then Skip=0
	EntityParent Bone(rd,SetBone(count)),RDMesh(rd,count-Skip)
Next

; ### Parent RagDoll Mesh To Body (Upper) Mesh

EntityParent RagDollMesh(rd),RDMesh(rd,1)

End Function

; ###################################################################################################

; ### 0 - Bone Not Used - Transparent
; ### 1 - Bone Not Used - Blue / Purple
; ### 2 - Bone Used For ODE Joint - Green
; ### 3 - Bone Highlight - White

.NodeData

Data 0,0,0,0,0,0,1,2,1,2,2,2,2,2,2,1,2,2,2,1,2,2,2,1

; ###################################################################################################