; ###################################################################################################
; #                                     JV-ODE - Ray Pick Demo                                      #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE - Ray Pick Demo"

Graphics3D 800,600,0,2

Global RayGeom
Global RayCollision=0

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.95)
dContactSetMu(48)

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global Camera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,10,-14
RotateEntity Camera,20,0,0

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ### Create Cylinder

Global Cylinder=CreateCylinder()
ScaleMesh Cylinder,0.2,3,0.2
PositionEntity Cylinder,0,3,-5
RotateEntity Cylinder,90,0,0
EntityColor Cylinder,230,0,0
EntityShininess Cylinder,0.7

; ###################################################################################################

AddObject()

While Not KeyHit(1)

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 0,70,"Cylinder = Ray Geom Position"
	Text 0,95,"Green = Collision"
	Text 0,110,"Red = No Collision"

	AddRay()

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function AddObject()

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
dBodySetPosition(ode\body,0,14,0)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateSphere(Space,3)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateSphere()
ScaleMesh ode\mesh,3,3,3
EntityColor ode\mesh,255,255,255
EntityShininess ode\mesh,0.7
EntityAlpha ode\mesh,0.8

End Function

; ###################################################################################################

Function AddRay()

RayGeom=dCreateRay(Space,3)
dGeomRaySet(RayGeom,0,3,-5,0,0,1)

; ### Perform Ray Collision Check
dSpaceCollide(Space,World,ContactGroup)
RayCollision=dGeom1CountCollisions(RayGeom)+dGeom2CountCollisions(RayGeom)
dJointGroupEmpty(ContactGroup)

If RayCollision>0
	EntityColor Cylinder,0,200,100
	Else
	EntityColor Cylinder,200,0,100
	End If

dGeomDestroy(RayGeom)

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################
