; ###################################################################################################
; #                                      JV-ODE - Slider Demo                                       #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE - Slider Demo"

Graphics3D 800,600,0,2

Global SliderRange=30

Dim Sphere(2)
Dim Joint(2)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.0)
dContactSetMu(48)

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global Camera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,90,-100
RotateEntity Camera,30,0,0

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

; ### Create Slider

For count=1 To 2
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	Sphere(count)=ode\body
	dBodySetPosition(ode\body,-30,100-(count*30),0)
	dBodySetRotation(ode\body,0,0,0)
	dBodySetAutoDisableFlag(ode\body,1)
	ode\geom=dCreateSphere(Space,4)
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateSphere()
	ScaleMesh ode\mesh,4,4,4
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
	If count=1
		Joint(count)=dJointCreateBall(World,0)
		dJointAttach(Joint(count),0,Sphere(count))
		dJointSetBallAnchor(Joint(count),dBodyGetPositionX(Sphere(count)),dBodyGetPositionY(Sphere(count)),dBodyGetPositionZ(Sphere(count)))
		Else
		Joint(count)=dJointCreateSlider(World,0)
		dJointAttach(Joint(count),Sphere(count-1),Sphere(count))
		dJointSetSliderAxis(Joint(count),0,1,0)
		dJointSetSliderParam(Joint(count),dParamLoStop,-SliderRange)
		dJointSetSliderParam(Joint(count),dParamHiStop,SliderRange)
		End If
Next

; ###################################################################################################

While Not KeyHit(1)

	UpdateKeys()

	If MilliSecs()-Timer>700
		AddObject()
		Timer=MilliSecs()
		End If

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 0,55,"Slider Position: "+dJointGetSliderPosition(Joint(2))
	Text 0,70,"Slider Rate: "+dJointGetSliderPositionRate(Joint(2))
	Text 640,0,"Add Force To Slider"
	Text 640,15,"Using Arrow Keys"
	Text 640,55,"Add Motor Force"
	Text 640,70,"Using F1 & F2"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function AddObject()

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
dBodySetPosition(ode\body,30,80,0)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,10,5,10)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,5,2.5,5
EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
EntityShininess ode\mesh,0.7

End Function

; ###################################################################################################

Function UpdateKeys()

dBodyEnable(Sphere(2))

If KeyDown(59)
	dJointSetSliderParam(Joint(2),dParamVel,-3)
	dJointSetSliderParam(Joint(2),dParamFMax,100)
	End If

If KeyDown(60)
	dJointSetSliderParam(Joint(2),dParamVel,3)
	dJointSetSliderParam(Joint(2),dParamFMax,100)
	End If

If dJointGetSliderPosition(Joint(2))=SliderRange Or dJointGetSliderPosition(Joint(2))=-SliderRange
	dJointSetSliderParam(Joint(2),dParamVel,0)
	dJointSetSliderParam(Joint(2),dParamFMax,100)
	End If

If KeyDown(200) Then dBodyAddForce(Sphere(2),0,0,10)
If KeyDown(208) Then dBodyAddForce(Sphere(2),0,0,-10)
If KeyDown(203) Then dBodyAddForce(Sphere(2),-10,0,0)
If KeyDown(205) Then dBodyAddForce(Sphere(2),10,0,0)

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################
